#!/usr/bin/env python3

'''
Parses LaTeX files and creates list of commands used in `caption` environments.

Runs on extracted archives produced by `arxiv_tex_extract.py` script.

Author: Matthew Petroff <https://mpetroff.net/>
Year: 2020

This script is released into the public domain using the CC0 1.0 Public
Domain Dedication: https://creativecommons.org/publicdomain/zero/1.0/
'''

from pathlib import Path
import subprocess
import csv
from multiprocessing import Process
from joblib import Parallel, delayed
from TexSoup import TexSoup

N_JOBS = 4

in_path = Path('tex')
out_path = Path('cmds')

def extract_cmds(arxiv_src, yymm):
    arxiv_id = arxiv_src.name
    print('Processing', arxiv_id)
    cmds = set()
    for tex_name in arxiv_src.glob('*.tex'):
        with open(tex_name) as doc:
            try:
                soup = TexSoup(doc, tolerance=1)
            except:
                continue
        for caption in soup.find_all('caption'):
            #cmds |= {str(c.name) for c in caption.children}
            new_cmds = []
            for c in caption.children:
                if c.name == 'BraceGroup':
                    new_cmds += [str(cc.name) for cc in c.children]
                else:
                    new_cmds.append(str(c.name))
            cmds |= set(new_cmds)
    if len(cmds) > 0:
        with open(out_path / yymm / (arxiv_id + '.txt'), 'w', newline='') as outfile:
            csv.writer(outfile).writerow(sorted(cmds))

def extract_cmds_wrapper(arxiv_src, yymm):
    proc = Process(target=extract_cmds, args=(arxiv_src, yymm))
    proc.start()
    proc.join(timeout=5)
    proc.terminate()

def process_src_dir(src_dir, yymm):
    dirn = src_dir / yymm
    (out_path / yymm).mkdir()
    Parallel(n_jobs=N_JOBS, prefer='threads')(delayed(extract_cmds_wrapper)(arxiv_src, yymm) for arxiv_src in sorted(dirn.glob('*')))

for yymm in sorted(in_path.glob('*')):
    process_src_dir(in_path, yymm.name)
