#!/usr/bin/env python3

'''
Looks for figure caption visual indicator candidates in LaTeX source of arXiv
pre-prints. Produces one HTML file per year with paper candidates, including
source of figure captions with candidates, with candidate commands in bold.

Runs on lists of commands in `caption` environments produced by
`arxiv_check_local.py` script in combination with extracted archives produced
by `arxiv_tex_extract.py` script.

Author: Matthew Petroff <https://mpetroff.net/>
Year: 2020

This script is released into the public domain using the CC0 1.0 Public
Domain Dedication: https://creativecommons.org/publicdomain/zero/1.0/
'''

import csv
from pathlib import Path
import html
from joblib import Parallel, delayed
from TexSoup import TexSoup

N_JOBS = 4

GENERIC = ['color', 'colour', 'swatch', 'marker', 'includegraphics', 'tikz', 'symbol', 'inlinegraphics', 'picture', 'draw', 'highlight'] # Exclude 'line', since it's part of too many words
COLORS = ['blue', 'brown', 'orange', 'red', 'yellow', 'pink', 'purple', 'gray', 'grey', 'magenta', 'violet']
SHAPES = ['square', 'triangle', 'diamond', 'pentagon', 'lozenge'] # Exclude 'circle', since it's too common for circling things
CIRCLES = ['opencircle', 'fullcircle', 'solidcircle', 'leftcircle', 'filledsmallcircle', 'filledcircle', 'textbigcircle', 'solidcirc', 'bigcirc', 'fullcirc']
LINES = ['dashed', 'solid', 'dotted', 'solidline', 'dashdot', 'dotdash', 'dashline', 'dotline', 'thickline']

KEYWORDS = GENERIC + COLORS + SHAPES + CIRCLES + LINES

base_path = Path('cmds')
tex_path = Path('tex')
html_path = Path('html')

def parse_paper(paper):
    paper_name = paper[5:]
    print('Processing', paper_name)
    arxiv_src = tex_path / paper
    new_html = ''
    for tex_name in arxiv_src.glob('*.tex'):
        with open(tex_name) as doc:
            try:
                soup = TexSoup(doc, tolerance=1)
            except:
                continue
        for caption in soup.find_all('caption'):
            new_cmds = []
            for c in caption.children:
                if len(c.args) > 0 and len(str(c.args[0])) >= 7:
                    s = str(c.args[0])
                    if s[:7] == '{black}' or s[:7] == '{white}':
                        # People generally force the text to black / white for other reasons
                        continue
                if c.name == 'BraceGroup':
                    for cc in c.children:
                        if len(cc.args) > 0 and len(str(cc.args[0])) >= 7:
                            s = str(cc.args[0])
                            if s[:7] == '{black}' or s[:7] == '{white}':
                                continue
                        new_cmds.append(str(cc.name))
                    #new_cmds += [str(cc.name) for cc in c.children]
                else:
                    new_cmds.append(str(c.name))
            new_cmds = sorted(set(new_cmds))
            keyword_matches = []
            for cmd in new_cmds:
                for keyword in KEYWORDS:
                    if keyword in cmd:
                        keyword_matches.append(cmd)
            if len(keyword_matches) > 0:
                #import pdb; pdb.set_trace()
                caption_text = str(caption)
                caption_text = html.escape(caption_text)
                for keyword in keyword_matches:
                    caption_text = caption_text.replace('\\' + keyword, '\\<b>' + keyword + '</b>')
                new_html += '<p>' + caption_text + '</p>'
    return paper, new_html

def process_year(yy):
    cmds = {}

    for p in sorted(base_path.glob(yy + '*/*.txt')):
        with open(p) as infile:
            cmd_list = list(csv.reader(infile))
            if len(cmd_list) > 0:
                for cmd in cmd_list[0]:
                    cmd = cmd.lower()
                    if len(cmd) < 1 or len(cmd) > 20 or ' ' in cmd or '$' in cmd or \
                        ',' in cmd or '.' in cmd or '-' in cmd or "'" in cmd or \
                        '"' in cmd or '`' in cmd or ':' in cmd:
                        # Probably a parsing mistake (or math)
                        continue
                    if cmd not in cmds:
                        cmds[cmd] = []
                    cmds[cmd].append(str(p)[len(str(base_path))+1:-4])

    papers = []
    for cmd in cmds:
        for keyword in KEYWORDS:
            if keyword in cmd:
                papers += cmds[cmd]
                continue

    papers = sorted(set(papers))

    html_doc = '''<!DOCTYPE html>
<head>
<meta charset="utf-8">
<title>Paper candidates</title>
<style>
p {
    max-width: 800px;
    font-family: monospace;
}
</style>
</head>
<body>
'''

    html_doc += '<h1>' + ('20' if yy < '50' else '19') + yy + '</h1>'

    paper_html = Parallel(n_jobs=N_JOBS)(delayed(parse_paper)(paper) for paper in papers)

    i = 1
    for paper in sorted(paper_html, key=lambda i: i[0]):
        paper_name = paper[0][5:]
        if '.' not in paper_name:
            # Old style
            # Re-add slash
            paper_name = paper_name[:-7] + '/' + paper_name[-7:]
        if len(paper[1]) > 0:
            html_doc += f'<h2>{i} &mdash; <a href="https://arxiv.org/pdf/{paper_name}.pdf">{paper_name}</a></h2>\n'
            html_doc += paper[1]
            i += 1

    html_doc += '''
</body>
</html>
'''

    with open(html_path / (yy + '.html'), 'w') as outfile:
        outfile.write(html_doc)


for year in range(92, 100):
    process_year(f'{year:02d}')

for year in range(0, 21):
    process_year(f'{year:02d}')
