#!/usr/bin/env python3

'''
Downloads, extracts, and repackages TeX source for arXiv pre-prints.

`AWS_ACCESS_KEY_ID` and `AWS_SECRET_ACCESS_KEY` environment variables need to
be set before running this script.

To minimize data egress charges, this should be run on an AWS EC2 instance in
the same region as the arXiv S3 bucket (us-east-1).

Author: Matthew Petroff <https://mpetroff.net/>
Year: 2020

This script is released into the public domain using the CC0 1.0 Public
Domain Dedication: https://creativecommons.org/publicdomain/zero/1.0/
'''

import tempfile
from pathlib import Path
import subprocess
import xml.etree.ElementTree as ET
import boto3

manifest = 'arXiv_src_manifest.xml'
s3 = boto3.client('s3')
out_path = Path('tex')

def process_src_archive(archive_name, yymm):
    td = tempfile.TemporaryDirectory()
    tdp = Path(td.name)
    fn = tdp / archive_name[4:]

    s3.download_file('arxiv', archive_name, str(fn), ExtraArgs={'RequestPayer': 'requester'})

    subprocess.run(['tar', '-C', tdp, '-xf', fn])

    dirn = tdp / yymm
    (out_path / yymm).mkdir()
    for arxiv_src in sorted(dirn.glob('*.gz')):
        arxiv_id = arxiv_src.stem
        print('Processing', arxiv_id)
        cd = dirn / arxiv_id
        cd.mkdir()
        tex_out = out_path / yymm / arxiv_id
        try:
            subprocess.run(['tar', '-C', cd, '-zxf', arxiv_src]).check_returncode()
        except subprocess.CalledProcessError:
            stdout = subprocess.run(['file', arxiv_src], stdout=subprocess.PIPE).stdout
            if b'gzip compressed data' in stdout and b'.tex",' in stdout:
                # Gzipped TeX file
                tex_out.mkdir()
                arxiv_src.rename(tex_out / 'texonly.tex.gz')
                subprocess.run(['gunzip', tex_out / 'texonly.tex.gz'])
        else:
            tex_out.mkdir()
            for tex_name in cd.glob('*.tex'):
                tex_name.rename(tex_out / tex_name.name)

    td.cleanup()

s3.download_file('arxiv', 'src/' + manifest, manifest, ExtraArgs={'RequestPayer': 'requester'})
for event, elem in ET.iterparse(manifest):
    if event == 'end' and elem.tag == 'filename':
        archive_name = elem.text
        yymm = archive_name[14:18]
        if yymm != '2007':
            continue
        process_src_archive(archive_name, yymm)
        subprocess.run(['tar', '-I', 'zstd', '-cf', out_path / (archive_name[14:] + '.zst'), out_path / yymm])
        subprocess.run(['rm', '-rf', out_path / yymm])
